@@ -21,7 +21,7 @@ __all__ = [
     "ReferenceError", "UnknownReferenceError", "BadReferenceError", "ParseError", "Grammar",
     "Terminal",
     "Literal", "Word", "Repetition", "ListRepetition", "Reference",
-    "GRAMMAR", "G", "ANY", "EMPTY", "REF", "LITERAL", "L", "OR", "EXCEPT", "WORD", "REPEAT", "LIST_OF", "OPTIONAL",
+    "GRAMMAR", "G", "ANY", "EMPTY", "REF", "LITERAL", "L", "OR", "EXCEPT", "WORD", "REPEAT", "LIST_OF", "OPTIONAL", "NOT_FOLLOWED_BY",
     "ZERO_OR_MORE", "ONE_OR_MORE", "ANY_EXCEPT", "BOL", "EOL", "EOF",
     "REST_OF_LINE", "SPACE",
     "generate_ebnf",
@@ -57,6 +57,12 @@ class InternalError (Exception):
   """
   pass
 
+class GrammarDefError (Exception):
+  """
+  This exception is raised when creating/defining new grammar classes if there is a problem with the definition which cannot be resolved.
+  """
+  pass
+
 class ReferenceError (Exception):
   """
   This is the base class for :exc:`UnknownReferenceError` and :exc:`BadReferenceError`.  It can be used to easily catch either exception.
@@ -345,6 +351,16 @@ class GrammarParser:
           # string, so ignore the error.
           return (None, None)
         errpos, expected = obj
+        if errpos == len(self.text.string) and self.grammar.grammar_whitespace:
+	  # If we hit EOF and this grammar is whitespace-consuming, check to
+	  # see whether we had only whitespace before the EOF.  If so, treat
+	  # this like the pos == len(self.text.string) case above.
+          whitespace_re = self.grammar.grammar_whitespace
+          if whitespace_re is True:
+            whitespace_re = util._whitespace_re
+          m = whitespace_re.match(self.text.string, pos)
+          if m and m.end() == len(self.text.string):
+            return (None, None)
         char = self.char + errpos
         line, col = util.calc_line_col(self.text.string, errpos, self.line, self.col, self.tabs)
         raise ParseError(self.grammar, self.text.string, errpos, char, line=line, col=col, expected=expected)
@@ -364,9 +380,9 @@ class GrammarParser:
     elif matchtype == 'last':
       count, obj = matches[-1]
     elif matchtype == 'longest':
-      count, obj = max(matches)
+      count, obj = max(matches, key=lambda m: m[0])
     elif matchtype == 'shortest':
-      count, obj = min(matches)
+      count, obj = min(matches, key=lambda m: m[0])
     elif matchtype == 'all':
       objs = [x[1] for x in matches]
       count = max(x[0] for x in matches)
@@ -396,7 +412,8 @@ class GrammarParser:
       if count is None:
         # Partial match
         break
-      self.update_pos(count)
+      elif matchtype != 'all':
+        self.skip(count)
       yield obj
       if not count:
 	# We matched a zero-length string.  If we keep looping, we'll just loop
@@ -433,7 +450,7 @@ class GrammarParser:
           "shortest"
             The match which uses up the shortest portion of the input text.
           "all"
-            Return all possible matches, in a list.
+            Return all possible matches, in a list.  Note that in this case the buffer position will not be automatically advanced.  You must call :func:`~GrammarParser.skip` manually.
       *bol*
         Treat the input text as starting at the beginning of a line (for the purposes of matching the :const:`BOL` grammar element).  It is not usually necessary to specify this explicitly.
     """
@@ -454,6 +471,8 @@ class GrammarParser:
     Attempt to match a list (or actually any iterable) of strings against the associated grammar.  This is effectively the same as calling :meth:`parse_string` repeatedly for each string in the list to obtain all matches in sequence.
 
     Return values, exceptions, and optional parameters are all exactly the same as for :meth:`parse_string`.
+
+    Note: Be careful using ``matchtype="all"`` with parse_lines/parse_file.  You must manually call :func:`~GrammarParser.skip` after each yielded match, or you will end up with an infinite loop!
     """
     if reset:
       self.reset()
@@ -472,6 +491,8 @@ class GrammarParser:
     Open and process the contents of a file using the associated grammar.  This is basically the same as opening the specified file, and passing the resulting file object to :meth:`parse_lines`.
 
     Return values, exceptions, and optional parameters are all exactly the same as for :meth:`parse_string`.
+
+    Note: Be careful using ``matchtype="all"`` with parse_lines/parse_file.  You must manually call :func:`~GrammarParser.skip` after each yielded match, or you will end up with an infinite loop!
     """
     if isinstance(file, str):
       with open(file, "r") as f:
@@ -481,8 +502,14 @@ class GrammarParser:
       for result in self.parse_lines(file, bol=bol, eof=eof, reset=reset, data=data, matchtype=matchtype):
         yield result
 
-  def update_pos(self, count):
+  def skip(self, count):
+    """
+    Skip forward the specified number of characters in the input buffer (discarding the text skipped over).
+    """
+
     if count:
+      if count > len(self.text.string):
+        raise ValueError("Attempt to skip past end of available buffer.")
       # The state may contain index values in it, which will become invalid if
       # we change the starting point, so we (unfortunately) need to nuke it.
       self.state = (None, None)
@@ -500,7 +527,7 @@ class GrammarParser:
 #                            Base (public) Classes                            #
 ###############################################################################
 
-class Grammar(metaclass=GrammarClass):
+class Grammar (metaclass=GrammarClass):
   """
   This class is not intended to be instantiated directly.  Instead, it is a base class to be used for defining your own custom grammars.
 
@@ -515,6 +542,7 @@ class Grammar(metaclass=GrammarClass):
   grammar_greedy = True
   grammar_null_subtoken_ok = True
   grammar_whitespace = None
+  grammar_error_override = False
   grammar_hashattrs = ('grammar_name', 'grammar', 'grammar_min', 'grammar_max', 'grammar_collapse', 'grammar_greedy', 'grammar_whitespace')
 
   @classmethod
@@ -558,6 +586,7 @@ class Grammar(metaclass=GrammarClass):
     positions = []
     best_error = None
     pos = index
+    first_pos = None
 
     while True:
       # Forward ho!
@@ -582,6 +611,8 @@ class Grammar(metaclass=GrammarClass):
             if pos < len(text.string) or text.eof:
               break
             text = yield (None, None)
+        if first_pos is None:
+          first_pos = pos
         s = grammar[len(objs)].grammar_parse(text, pos, sessiondata)
         offset, obj = next(s)
         while offset is None:
@@ -601,8 +632,8 @@ class Grammar(metaclass=GrammarClass):
       # find something else to follow...
       while True:
         if greedy and len(objs) >= grammar_min:
-	  # If we are greedy, then return matches only after we"ve gone as far
-	  # forward as possible, while we"re backtracking (returns the longest
+	  # If we are greedy, then return matches only after we've gone as far
+	  # forward as possible, while we're backtracking (returns the longest
 	  # matches first)
           yield (pos - index, cls(text.string, index, pos, objs))
 	  # We need to copy objs for any further stuff, since it's now part of
@@ -632,7 +663,21 @@ class Grammar(metaclass=GrammarClass):
       # If so, give up.  If not, loop around and try going forward again.
       if not states:
         break
-    yield error_result(*best_error)
+    if cls.grammar_error_override:
+      # If our sub-grammars failed to match, but we've got
+      # grammar_error_override set, return ourselves as the failed match
+      # grammar instead.
+      yield error_result(index, cls)
+    elif ((len(cls.grammar) == 1)
+         and (best_error[0] == first_pos)
+         and (cls.grammar_desc != cls.grammar_name) ):
+      # We're just a simple wrapper (i.e. an alias) around another single
+      # grammar class, and it failed to match, and we have a custom
+      # grammar_desc.  Return ourselves as the failed match grammar so the
+      # ParseError will contain our grammar_desc instead.
+      yield error_result(index, cls)
+    else:
+      yield error_result(*best_error)
 
   @classmethod
   def grammar_ebnf_lhs(cls, opts):
@@ -904,7 +949,7 @@ class Grammar(metaclass=GrammarClass):
     return self.elements[index]
 
   def __len__(self):
-    return len(self.elements)
+    return len(self.string)
 
   def __bool__(self):
     return bool(self.elements) or self.grammar_terminal
@@ -1079,6 +1124,16 @@ def OR(*grammars, **kwargs):
 class OR_Operator (Grammar):
   grammar_whitespace = False
 
+  @classmethod
+  def __class_init__(cls, attrs):
+    if not "grammar_desc" in attrs and cls.grammar:
+      # This is not used directly when constructing ParseExceptions (because we
+      # never return ourselves as a failure class, we only return the failures
+      # from our subgrammars), but is needed by some other grammars which
+      # construct their grammar_desc based on their subgrammar's grammar_descs
+      # (i.e. NOT())
+      cls.grammar_desc = " or ".join(g.grammar_desc for g in cls.grammar)
+
   @classmethod
   def grammar_parse(cls, text, index, sessiondata):
     best_error = None
@@ -1116,6 +1171,69 @@ class OR_Operator (Grammar):
   def grammar_ebnf_rhs(cls, opts):
     return None
 
+def NOT_FOLLOWED_BY(*grammar, **kwargs):
+  """
+  Returns a successful match as long as the next text after this point does NOT match the specified grammar.
+
+  When successful (that is, the next text in the input does not match the specified grammar), this element of the parse tree will contain :const:`None`, and no input text will be consumed.  When unsuccessful (that is, the next text does match), a :exc:`ParseError` will be raised.
+  """
+  cdict = util.make_classdict(NotFollowedBy, grammar, kwargs)
+  return GrammarClass("<NOT_FOLLOWED_BY>", (NotFollowedBy,), cdict)
+
+class NotFollowedBy (Grammar):
+  grammar_whitespace = False
+  grammar_collapse = True
+
+  @classmethod
+  def __class_init__(cls, attrs):
+    if not cls.grammar:
+      cls.grammar = (EMPTY,)
+    else:
+      cls.grammar = (GRAMMAR(cls.grammar),)
+    if not "grammar_desc" in attrs and cls.grammar:
+      cls.grammar_desc = "anything except {}".format(cls.grammar[0].grammar_desc)
+
+  @classmethod
+  def grammar_parse(cls, text, index, sessiondata):
+    best_error = None
+    g = cls.grammar[0]
+    results = g.grammar_parse(text, index, sessiondata)
+    count, obj = next(results)
+    while count is None:
+      if text.eof:
+        # Subgrammars should not be asking for more data after eof.
+        raise InternalError("{} requested more data when at EOF".format(g))
+      text = yield (None, None)
+      count, obj = results.send(text)
+    if count is not False:
+      # The subgrammar matched.  That means we should consider this a parse
+      # error.
+      yield error_result(index, cls)
+    else:
+      # Subgrammar did not match.  Return a (successful) None match.
+      yield (0, cls(''))
+    
+  @classmethod
+  def grammar_details(cls, depth=-1, visited=None):
+    if not visited:
+      visited = (cls,)
+    elif cls in visited:
+      # Circular reference.  Stop here.
+      return cls.grammar_name
+    else:
+      visited = visited + (cls,)
+    return "NOT_FOLLOWED_BY({})".format(cls.grammar[0].grammar_details(depth, visited))
+
+  @classmethod
+  def grammar_ebnf_lhs(cls, opts):
+    sub_lhs, sub_nts = cls.grammar[0].grammar_ebnf_lhs(opts)
+    desc = "not followed by {}".format(sub_lhs)
+    return (util.ebnf_specialseq(cls, opts, desc=desc), (cls.grammar[0],))
+
+  @classmethod
+  def grammar_ebnf_rhs(cls, opts):
+    return None
+
 def EXCEPT(grammar, exc_grammar, **kwargs):
   """
   Match *grammar*, but only if it does not also match *exception_grammar*.  (This is equivalent to the ``-`` (exception) operator in EBNF) :func:`EXCEPT` grammars can also be created by combining other grammars in python expressions using the except operator (``-``).
@@ -1171,7 +1289,8 @@ class ExceptionGrammar (Grammar):
     # obvious there were extra conditions on the match that weren't fulfilled.
     if best_error[0] == index:
       yield error_result(index, cls)
-    yield error_result(*best_error)
+    else:
+      yield error_result(*best_error)
     
   @classmethod
   def grammar_details(cls, depth=-1, visited=None):
@@ -1595,6 +1714,8 @@ def generate_ebnf(grammar, **opts):
       The number of characters that subsequent (wrapped) lines should be indented.  Can be set to either a number or to :const:`True`.  If set to :const:`True`, the indent will be auto-calculated to line up with the position of the RHS in the first line.
     *expand_terminals* (default False)
       If grammars have subgrammars, show their expansion even if :attr:`~Grammar.grammar_terminal` is true.
+    *special_style* (default "desc")
+      How some grammars (which can't be easily represented as EBNF) should be represented inside EBNF "special sequences".  Valid options are "desc" (use the (human-readable) :attr:`~Grammar.grammar_desc` text), "name" (just use the grammar's name), or "python" (use a repr-like syntax similar to the python syntax used to create them).
 
   Additional options may also be offered by certain individual grammars.
   """
